﻿using System;
using System.IO;
using System.IO.IsolatedStorage;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Xml.Serialization;
using Infragistics.Controls.Schedules;
using SilverlightXamScheduleApp.ViewModel;

namespace SilverlightXamScheduleApp
{
    public partial class MainPage : UserControl
    {


        #region Constants
        const string IsoFile = "Falorites.xml";
        #endregion //Constants

        #region Member Vaiables
        private bool _firstClickDone = false;

        public DateTime _lastClick = DateTime.Now;

        private ScheduleViewModel _scheduleViewModel;
        Point _clickPosition;
        #endregion //Member Vaiables



        #region Constructors
        public MainPage()
        {
            InitializeComponent();

            _scheduleViewModel = new ScheduleViewModel("currentUser", "Current User", "currentUserCalendar");
            this.DataContext = this._scheduleViewModel;
        }
        #endregion //Constructors

        #region Methods

        #region CleanStorage
        private static void CleanStorage(IsolatedStorageFile store)
        {
            String[] dirNames = store.GetDirectoryNames("*");
            String[] fileNames = store.GetFileNames("\\*");

            if (fileNames.Length > 0)
            {
                for (int i = 0; i < fileNames.Length; ++i)
                {
                    // Delete the files.
                    store.DeleteFile("\\" + fileNames[i]);
                }
                // Confirm that no files remain.
                fileNames = store.GetFileNames("\\*");
            }

            if (dirNames.Length > 0)
            {
                for (int i = 0; i < dirNames.Length; ++i)
                {
                    // Delete the Archive directory.
                    store.DeleteDirectory(dirNames[i]);
                }
            }
        }
        #endregion //CleanStorage

        #region InitResourceCalendar
        private void InitResourceCalendar(ResourceCalendarInfo calendar)
        {

                var resource = xamScheduleDataManager1.ResourceItems.GetResourceFromId(this._scheduleViewModel.Resources[0].Id);

                if (resource == null)
                {
                    return;
                }

                ResourceCalendar calendar2 = resource.Calendars.First(c => c.Id == calendar.Id);

                CalendarGroup groupWithCalendarAdded =
                    xamScheduleDataManager1.CalendarGroups.FirstOrDefault(cg => cg.Calendars.Contains(calendar2));
                if (groupWithCalendarAdded != null)
                {
                    groupWithCalendarAdded.Calendars.Remove(calendar2);
                    if (groupWithCalendarAdded.Calendars.Count == 0)
                        xamScheduleDataManager1.CalendarGroups.Remove(groupWithCalendarAdded);
                }

                CalendarGroup calGroup;
                if (xamScheduleDataManager1.CalendarGroups.Count > 0)
                {
                    int index = 0;
                    calGroup = xamScheduleDataManager1.CalendarGroups[index];
                    calGroup.Calendars.Add(calendar2);
                }
                else
                {
                    calGroup = new CalendarGroup();
                    calGroup.Calendars.Add(calendar2);
                    xamScheduleDataManager1.CalendarGroups.Add(calGroup);
                }

        }
        #endregion //InitResourceCalendar

        #region ReadDataFromStore
        //Reads a file from the Isolated Storage, specified by name
        private static byte[] ReadDataFromStore(string fileName)
        {
            using (var store = IsolatedStorageFile.GetUserStoreForApplication())
            {
                IsolatedStorageFileStream fileStream;
                fileStream = store.OpenFile(fileName, FileMode.Open);
                byte[] b = new byte[fileStream.Length];
                fileStream.Read(b, 0, (int)fileStream.Length);
                fileStream.Close();
                return b;
            }
        }
        #endregion //ReadDataFromStore

        #region SetCurrentGroup
        private void SetCurrentGroup(FrameworkElement sender)
        {
            ResourceCalendarInfo calendarInfo = sender.DataContext as ResourceCalendarInfo;
            if (calendarInfo != null)
            {

                this._scheduleViewModel.CurrentResourceCalendar = calendarInfo;


                var resource = xamScheduleDataManager1.ResourceItems.GetResourceFromId(this._scheduleViewModel.Resources[0].Id);

                ResourceCalendar calendar2 = resource.Calendars.First(c => c.Id == calendarInfo.Id);
                this.xamScheduleDataManager1.CalendarGroups[0].SelectedCalendar = calendar2;
            }
        }
        #endregion //SetCurrentGroup

        #region SetScheduleMode
        private void SetScheduleMode(string viewName)
        {
            if (this.xamDayView1 == null || this.xamMonthView1 == null || this.xamScheduleView1 == null)
            {
                return;
            }

            switch (viewName)
            {
                case "DayView":
                    this.xamDayView1.Visibility = System.Windows.Visibility.Visible;
                    this.xamMonthView1.Visibility = System.Windows.Visibility.Collapsed;
                    this.xamScheduleView1.Visibility = System.Windows.Visibility.Collapsed;
                    break;
                case "ScheduleView":
                    this.xamDayView1.Visibility = System.Windows.Visibility.Collapsed;
                    this.xamMonthView1.Visibility = System.Windows.Visibility.Collapsed;
                    this.xamScheduleView1.Visibility = System.Windows.Visibility.Visible;
                    break;
                case "MonthView":
                    this.xamDayView1.Visibility = System.Windows.Visibility.Collapsed;
                    this.xamMonthView1.Visibility = System.Windows.Visibility.Visible;
                    this.xamScheduleView1.Visibility = System.Windows.Visibility.Collapsed;
                    break;
            }
        }
        #endregion //SetScheduleMode

        #region WriteDataToStore
        // Writes a file to an Isolated IStrorage, specified via its name 
        // and content like a byte array
        private static void WriteDataToStore(byte[] data, string fileName)
        {
            if (data == null || data.Length == 0)
                return;
            using (var store = IsolatedStorageFile.GetUserStoreForApplication())
            {
                CleanStorage(store);

                IsolatedStorageFileStream fileStream = store.CreateFile(fileName);
                fileStream.Write(data, 0, data.Length);
                fileStream.Close();

            }
        }

        #endregion //WriteDataToStore 

        #endregion //Methods

        #region EventHandlers

        #region Border_MouseLeftButtonDown
        private void Border_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            FrameworkElement element = sender as FrameworkElement;

            DateTime clickTime = DateTime.Now;

            TimeSpan span = clickTime - _lastClick;

            if (span.TotalMilliseconds > 300 || _firstClickDone == false)
            {


                _clickPosition = e.GetPosition(element);

                _firstClickDone = true;

                _lastClick = DateTime.Now;

            }

            else
            {

                Point position = e.GetPosition(element);

                if (Math.Abs(_clickPosition.X - position.X) < 4 &&

                    Math.Abs(_clickPosition.Y - position.Y) < 4)
                {

                    SetCurrentGroup(element);
                }

                _firstClickDone = false;

            }
        }
        #endregion //Border_MouseLeftButtonDown

        #region BtnDeleteCalendars_Click
        private void BtnDeleteCalendars_Click(object sender, RoutedEventArgs e)
        {
            this._scheduleViewModel.ResourceCalendars.Clear();

        }
        #endregion //BtnDeleteCalendars_Click

        #region BtnLoadData_Click
        private void BtnLoadData_Click(object sender, RoutedEventArgs e)
        {

            XmlSerializer xs = new XmlSerializer(typeof(ScheduleViewModel));

            Stream str = new MemoryStream(ReadDataFromStore(IsoFile));
            using (StreamReader rd = new StreamReader(str))
            {
                ScheduleViewModel favourites = xs.Deserialize(rd) as ScheduleViewModel;
                if (favourites != null)
                {
                    this._scheduleViewModel.Appointments.Clear();
                    this._scheduleViewModel.ResourceCalendars.Clear();

                    foreach (ResourceCalendarInfo calInfo in favourites.ResourceCalendars)
                    {
                        this._scheduleViewModel.ResourceCalendars.Add(calInfo);
                        this.InitResourceCalendar(calInfo);
                    }

                    foreach (AppointmentInfo app in favourites.Appointments)
                    {
                        this._scheduleViewModel.Appointments.Add(app);
                    }
                }
            }
        }
        #endregion //BtnLoadData_Click

        #region BtnSaveData_Click
        private void BtnSaveData_Click(object sender, RoutedEventArgs e)
        {
            XmlSerializer xs = new XmlSerializer(typeof(ScheduleViewModel));
            MemoryStream str = new MemoryStream();
            StreamWriter sr = new StreamWriter(str);
            xs.Serialize(sr, this._scheduleViewModel);
            WriteDataToStore(str.ToArray(), IsoFile);
        }
        #endregion //BtnSaveData_Click

        #region BtnCreateCalendar_Click
        private void BtnCreateCalendar_Click(object sender, RoutedEventArgs e)
        {
            int count = _scheduleViewModel.ResourceCalendars.Count+1;
            ResourceCalendarInfo calendar = new ResourceCalendarInfo
            {
                Id = "Calendar" + count.ToString(),
                OwningResourceId = _scheduleViewModel.Resources[0].Id
            };
            this.AddCalendarWindow.DataContext = calendar;
            
            this.AddCalendarWindow.Show();

        }
        #endregion //BtnCreateCalendar_Click 

        #region BtnSubmit_Click
        private void BtnSubmit_Click(object sender, RoutedEventArgs e)
        {

            ResourceCalendarInfo calendar = AddCalendarWindow.DataContext as ResourceCalendarInfo;
            if (calendar != null)
            {
                if (this._scheduleViewModel.ResourceCalendars.Count == 1)
                {
                    InitResourceCalendar(this._scheduleViewModel.ResourceCalendars[0]);
                }

                _scheduleViewModel.ResourceCalendars.Add(calendar);

                InitResourceCalendar(calendar);
            }
            this.AddCalendarWindow.Close();
        }
        #endregion //BtnSubmit_Click

        #region CmbCalendarDisplayModeSelectionChanged
        private void CmbCalendarDisplayModeSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (e.AddedItems.Count < 1)
            {
                return;
            }

            CalendarDisplayMode cmode = (CalendarDisplayMode)e.AddedItems[0];

            this.xamDayView1.CalendarDisplayMode = cmode;
            this.xamScheduleView1.CalendarDisplayMode = cmode;
            this.xamMonthView1.CalendarDisplayMode = cmode;
        }
        #endregion //CmbCalendarDisplayModeSelectionChanged

        #region ListScheduleDataConnector1Loaded
        private void ListScheduleDataConnector1Loaded(object sender, RoutedEventArgs e)
        {

            listScheduleDataConnector1.TimeZoneInfoProviderResolved.LocalTimeZoneId = "Jordan Standard Time";

        }
        #endregion //ListScheduleDataConnector1Loaded

        #region ListScheduleViews_SelectionChanged
        private void ListScheduleViews_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            ListBoxItem item = e.AddedItems[0] as ListBoxItem;
            if (item != null)
            {
                SetScheduleMode(item.Tag.ToString());
            }
        }
        #endregion //ListScheduleViews_SelectionChanged

        #endregion //EventHandlers


    }
}
